@@ -2,12 +2,19 @@ import pytest
 import time
 import datetime
 import loguru
-from pytest_mock import MockerFixture
+from pytest_mock import MockFixture, MockerFixture
 import shioaji as sj
 
 from decimal import Decimal
 from dataclasses import dataclass
-from sjtrade.trader import Position, PositionCond, SJTrader, SimulationShioaji
+from sjtrade.position import PriceSet
+from sjtrade.trader import (
+    Position,
+    PositionCond,
+    SJTrader,
+    SimulationShioaji,
+    StrategyBasic,
+)
 from shioaji.constant import (
     Action,
     TFTStockPriceType,
@@ -29,65 +36,81 @@ class TickSTKv1:
     simtrade: bool
 
 
-@pytest.fixture
-def sjtrader(api: sj.Shioaji) -> SJTrader:
-    return SJTrader(api)
-
-
-@pytest.fixture
-def sjtrader_sim(api: sj.Shioaji) -> SJTrader:
-    return SJTrader(api, simulation=True)
-
-
 @pytest.fixture
 def positions():
     return {"1605": -1, "6290": -3, "0000": 3}
 
 
 @pytest.fixture
-def sjtrader_entryed(sjtrader: SJTrader, positions: dict):
-    sjtrader.api.place_order.side_effect = lambda contract, order: (
+def sjtrader(api: sj.Shioaji, mocker: MockFixture, positions: dict) -> SJTrader:
+    sjtrader = SJTrader(api)
+    sjtrader.stratagy = StrategyBasic(entry_pct=0.05, contracts=api.Contracts)
+    sjtrader.stratagy.read_position_func = mocker.MagicMock()
+    sjtrader.stratagy.read_position_func.return_value = positions
+    return sjtrader
+
+
+@pytest.fixture
+def sjtrader_sim(api: sj.Shioaji, mocker: MockFixture, positions: dict) -> SJTrader:
+    sjtrader = SJTrader(api, simulation=True)
+    sjtrader.stratagy = StrategyBasic(entry_pct=0.05, contracts=api.Contracts)
+    sjtrader.stratagy.read_position_func = mocker.MagicMock()
+    sjtrader.stratagy.read_position_func.return_value = positions
+    return sjtrader
+
+
+@pytest.fixture
+def sjtrader_entryed(sjtrader: SJTrader):
+    sjtrader.api.place_order.side_effect = lambda contract, order, timeout: (
         sj.order.Trade(
             contract,
             order,
             sj.order.OrderStatus(status=sj.order.Status.PreSubmitted),
         )
     )
-    res = sjtrader.place_entry_order(positions, 1.05)
+    res = sjtrader.place_entry_positions()
     return sjtrader
 
 
 @pytest.fixture
 def sjtrader_entryed_sim(sjtrader_sim: SJTrader, positions: dict) -> SJTrader:
-    res = sjtrader_sim.place_entry_order(positions, 1.05)
+    res = sjtrader_sim.place_entry_positions()
     return sjtrader_sim
 
 
 def test_sjtrader(api: sj.Shioaji):
     sjtrader = SJTrader(api)
+    sjtrader.stratagy = StrategyBasic()
     assert hasattr(sjtrader, "api")
     sjtrader.stop_profit_pct = 0.1
     assert sjtrader.stop_profit_pct == 0.1
+    assert sjtrader.stratagy.stop_profit_pct == 0.1
     sjtrader.stop_loss_pct = 0.1
     assert sjtrader.stop_loss_pct == 0.1
+    assert sjtrader.stratagy.stop_loss_pct == 0.1
     sjtrader.position_filepath = "pos.txt"
     assert sjtrader.position_filepath == "pos.txt"
+    assert sjtrader.stratagy.position_filepath == "pos.txt"
     sjtrader.entry_pct = 0.07
     assert sjtrader.entry_pct == 0.07
+    assert sjtrader.stratagy.entry_pct == 0.07
 
 
-def test_sjtrader_start(sjtrader: SJTrader, mocker: MockerFixture, positions: dict):
-    read_position_mock = mocker.patch("sjtrade.trader.read_position")
-    read_position_mock.return_value = positions
+def test_sjtrader_start(sjtrader: SJTrader, mocker: MockerFixture):
     sleep_until_mock = mocker.patch("sjtrade.trader.sleep_until")
     sjtrader.start()
-    read_position_mock.assert_called_once()
+    sjtrader.stratagy.read_position_func.assert_called_once()
     sjtrader.api.set_order_callback.assert_called_once_with(sjtrader.order_deal_handler)
     sjtrader.api.quote.set_on_tick_stk_v1_callback.assert_has_calls(
         [((sjtrader.cancel_preorder_handler,),), ((sjtrader.intraday_handler,),)]
     )
     sleep_until_mock.assert_has_calls(
-        [((8, 45),), ((8, 54, 59),), ((8, 59, 55),), ((13, 25, 59),)]
+        [
+            mocker.call(datetime.time(8, 45)),
+            mocker.call(datetime.time(8, 54, 59)),
+            mocker.call(datetime.time(8, 59, 55)),
+            mocker.call(datetime.time(13, 25, 59)),
+        ]
     )
 
 
@@ -97,15 +120,19 @@ def test_sjtrader_sj_event_handler(sjtrader: SJTrader, logger: loguru._logger.Lo
 
 
 def test_sjtrader_place_entry_order(
-    sjtrader: SJTrader, logger: loguru._logger.Logger, positions: dict
+    sjtrader: SJTrader,
+    logger: loguru._logger.Logger,
+    logger_stratagy: loguru._logger.Logger,
 ):
-    sjtrader.api.place_order.side_effect = lambda contract, order: sj.order.Trade(
-        contract, order, sj.order.OrderStatus(status=sj.order.Status.PreSubmitted)
+    sjtrader.api.place_order.side_effect = (
+        lambda contract, order, timeout: sj.order.Trade(
+            contract, order, sj.order.OrderStatus(status=sj.order.Status.PreSubmitted)
+        )
     )
     sjtrader.stop_loss_pct = 0.085
     sjtrader.stop_profit_pct = 0.09
-    res = sjtrader.place_entry_order(positions, 1.05)
-    logger.warning.assert_called_once()
+    res = sjtrader.place_entry_positions()
+    logger_stratagy.warning.assert_called_once()
     sjtrader.api.quote.subscribe.assert_has_calls(
         [
             ((sjtrader.api.Contracts.Stocks["1605"],), dict(version=QuoteVersion.v1)),
@@ -142,14 +169,38 @@ def test_sjtrader_place_entry_order(
             status=sj.order.OrderStatus(status=sj.order.Status.PreSubmitted),
         ),
     ]
+    cond_1605 = PositionCond(
+        quantity=-1,
+        entry_price=[
+            PriceSet(price=41.35, quantity=-1, price_type=TFTStockPriceType.LMT)
+        ],
+        stop_loss_price=[
+            PriceSet(price=42.7, quantity=-1, price_type=TFTStockPriceType.MKT)
+        ],
+        stop_profit_price=[
+            PriceSet(price=35.9, quantity=-1, price_type=TFTStockPriceType.MKT)
+        ],
+        cover_price=[],
+    )
+    cond_6290 = PositionCond(
+        quantity=-3,
+        entry_price=[
+            PriceSet(price=60.1, quantity=-3, price_type=TFTStockPriceType.LMT)
+        ],
+        stop_loss_price=[
+            PriceSet(price=62.1, quantity=-3, price_type=TFTStockPriceType.MKT)
+        ],
+        stop_profit_price=[
+            PriceSet(price=52.2, quantity=-3, price_type=TFTStockPriceType.MKT)
+        ],
+        cover_price=[],
+    )
+    assert sjtrader.positions["1605"].cond == cond_1605
+    assert sjtrader.positions["6290"].cond == cond_6290
     assert sjtrader.positions == {
         "1605": Position(
             contract=sjtrader.api.Contracts.Stocks["1605"],
-            cond=PositionCond(
-                quantity=-1,
-                stop_loss_price=42.7,
-                stop_profit_price=35.9,
-            ),
+            cond=cond_1605,
             entry_trades=[
                 expected[0],
             ],
@@ -158,11 +209,7 @@ def test_sjtrader_place_entry_order(
         ),
         "6290": Position(
             contract=sjtrader.api.Contracts.Stocks["6290"],
-            cond=PositionCond(
-                quantity=-3,
-                stop_loss_price=62.1,
-                stop_profit_price=52.2,
-            ),
+            cond=cond_6290,
             entry_trades=[
                 expected[1],
             ],
@@ -171,7 +218,7 @@ def test_sjtrader_place_entry_order(
         ),
     }
     assert len(res) == 2
-    assert res == expected
+    assert res == sjtrader.positions
 
 
 def test_sjtrader_cancel_preorder_handler(
@@ -189,7 +236,7 @@ def test_sjtrader_cancel_preorder_handler(
 
     sjtrader_entryed.cancel_preorder_handler(Exchange.TSE, tick)
     sjtrader_entryed.api.cancel_order.assert_called_once_with(
-        sjtrader_entryed.positions["1605"].entry_trades[0]
+        sjtrader_entryed.positions["1605"].entry_trades[0], timeout=0
     )
     assert logger.info.called
     # TODO need single trade update status interface
@@ -205,7 +252,7 @@ def test_sjtrader_re_entry_order(
 ):
     tick = TickSTKv1("1605", "2022-05-25 08:45:01", 43.3, True)
 
-    def make_cancel_order_status(trade):
+    def make_cancel_order_status(trade, timeout):
         trade.status.status = sj.order.Status.Cancelled
         trade.status.cancel_quantity = trade.order.quantity
 
@@ -228,6 +275,7 @@ def test_sjtrader_re_entry_order(
             first_sell=StockFirstSell.Yes,
             custom_field="dt1",
         ),
+        timeout=0,
     )
     assert logger.info.called
 
@@ -588,10 +636,12 @@ def test_sjtrader_sim(api: sj.Shioaji):
 
 
 def test_sjtrader_sim_place_entry_order(
-    sjtrader_sim: SJTrader, logger: loguru._logger.Logger, positions: dict
+    sjtrader_sim: SJTrader,
+    logger: loguru._logger.Logger,
+    logger_stratagy: loguru._logger.Logger,
 ):
-    res = sjtrader_sim.place_entry_order(positions, 1.05)
-    logger.warning.assert_called_once()
+    res = sjtrader_sim.place_entry_positions()
+    logger_stratagy.warning.assert_called_once()
     sjtrader_sim.api.quote.subscribe.assert_has_calls(
         [
             (
